/*
 *   Copyright (c) 2000 S. Noda (HZG02104@nifty.ne.jp)
 *
 *   Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose is hereby granted provided
 * that the above copyright notice and this permission notice appear
 * in all copies of the software and related documentation.
 *
 *                          NO WARRANTY
 *
 *   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES;
 * WITHOUT EVEN THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

#include "stdafx.h"
#include "wmzip.h"
#include "ZipDLL.h"
#include "StatusDlg.h"
#include "PasswordDlg.h"
#include "CommentDlg.h"
#include "ConfigDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static HINSTANCE hDll;

static ZP_VERSION fpZpVersion;
static ZP_INIT fpZpInit;
static ZP_SETOPTIONS fpZpSetOptions;
static ZP_GETOPTIONS fpZpGetOptions;
static ZP_ARCHIVE fpZpArchive;

static CStatusDlg* pDlg;

static ZIPUSERFUNCTIONS ZipUserFunctions;
static ZPOPT ZpOpt;
static ZCL ZpZCL;

static char Date[9], /*szRootDir[_MAX_PATH], */szTempDir[_MAX_PATH];

BOOL LoadZipLibrary()
{
	hDll = LoadLibrary("zip32.dll");
	
	if (hDll)
	{
		fpZpVersion = (ZP_VERSION) GetProcAddress(hDll, "ZpVersion");
		fpZpInit = (ZP_INIT) GetProcAddress(hDll, "ZpInit");
		fpZpSetOptions = (ZP_SETOPTIONS) GetProcAddress(hDll, "ZpSetOptions");
		fpZpGetOptions = (ZP_GETOPTIONS) GetProcAddress(hDll, "ZpGetOptions");
		fpZpArchive = (ZP_ARCHIVE) GetProcAddress(hDll, "ZpArchive");
		
		if (fpZpVersion && fpZpInit && fpZpSetOptions && fpZpGetOptions && fpZpArchive)
			return TRUE;
	}
	
	return FALSE;
}

BOOL CheckZipLibrary()
{
	ASSERT(fpZpVersion);
	
	ZpVer ver;
	(*fpZpVersion)(&ver);
	
	if (ver.zip.major >= 2 && ver.zip.minor >= 3)
		return TRUE;
	
	return FALSE;
}

int InitZipLibrary()
{
	ASSERT(fpZpInit);
	
	ZipUserFunctions.print = UpdateStatusDlg;
	ZipUserFunctions.password = PasswordPrompt;
	ZipUserFunctions.comment = CommentPrompt;
	
	return (*fpZpInit)(&ZipUserFunctions);
}

void FreeZipLibrary()
{
	if (hDll)
		FreeLibrary(hDll);
	
	hDll = NULL;
	
	fpZpVersion = NULL;
	fpZpInit = NULL;
	fpZpSetOptions = NULL;
	fpZpGetOptions = NULL;
	fpZpArchive = NULL;
}

void LoadZipOptions()
{
	ASSERT(fpZpGetOptions);
	
	CWinApp* pApp = AfxGetApp();
	
	ZpOpt = (*fpZpGetOptions)();
	
	lstrcpy(Date, pApp->GetProfileString("ZpOpt", "Date", "\0"));
	ZpOpt.Date = Date;
	//lstrcpy(szRootDir, pApp->GetProfileString("ZpOpt", "szRootDir", "\0"));
	//ZpOpt.szRootDir = szRootDir;
	lstrcpy(szTempDir, pApp->GetProfileString("ZpOpt", "szTempDir", "\0"));
	ZpOpt.szTempDir = szTempDir;
	ZpOpt.fTemp = pApp->GetProfileInt("ZpOpt", "fTemp", FALSE);
	ZpOpt.fSuffix = pApp->GetProfileInt("ZpOpt", "fSuffix", FALSE);
	ZpOpt.fEncrypt = pApp->GetProfileInt("ZpOpt", "fEncrypt", TRUE);
	ZpOpt.fSystem = pApp->GetProfileInt("ZpOpt", "fSystem", TRUE);
	ZpOpt.fVolume = pApp->GetProfileInt("ZpOpt", "fVolume", FALSE);
	ZpOpt.fExtra = pApp->GetProfileInt("ZpOpt", "fExtra", TRUE);
	ZpOpt.fNoDirEntries = pApp->GetProfileInt("ZpOpt", "fNoDirEntries", FALSE);
	ZpOpt.fExcludeDate = pApp->GetProfileInt("ZpOpt", "fExcludeDate", FALSE);
	ZpOpt.fIncludeDate = pApp->GetProfileInt("ZpOpt", "fIncludeDate", FALSE);
	ZpOpt.fVerbose = pApp->GetProfileInt("ZpOpt", "fVerbose", TRUE);
	ZpOpt.fQuiet = pApp->GetProfileInt("ZpOpt", "fQuiet", FALSE);
	ZpOpt.fCRLF_LF = pApp->GetProfileInt("ZpOpt", "fCRLF_LF", FALSE);
	ZpOpt.fLF_CRLF = pApp->GetProfileInt("ZpOpt", "fLF_CRLF", FALSE);
	ZpOpt.fJunkDir = pApp->GetProfileInt("ZpOpt", "fJunkDir", FALSE);
	ZpOpt.fGrow = pApp->GetProfileInt("ZpOpt", "fGrow", FALSE);
	ZpOpt.fForce = pApp->GetProfileInt("ZpOpt", "fForce", FALSE);
	ZpOpt.fMove = pApp->GetProfileInt("ZpOpt", "fMove", FALSE);
	ZpOpt.fDeleteEntries = pApp->GetProfileInt("ZpOpt", "fDeleteEntries", FALSE);
	ZpOpt.fUpdate = pApp->GetProfileInt("ZpOpt", "fUpdate", FALSE);
	ZpOpt.fFreshen = pApp->GetProfileInt("ZpOpt", "fFreshen", FALSE);
	ZpOpt.fJunkSFX = pApp->GetProfileInt("ZpOpt", "fJunkSFX", FALSE);
	ZpOpt.fLatestTime = pApp->GetProfileInt("ZpOpt", "fLatestTime", FALSE);
	ZpOpt.fComment = pApp->GetProfileInt("ZpOpt", "fComment", TRUE);
	ZpOpt.fOffsets = pApp->GetProfileInt("ZpOpt", "fOffsets", FALSE);
	ZpOpt.fPrivilege = pApp->GetProfileInt("ZpOpt", "fPrivilege", FALSE);
	//ZpOpt.fEncryption = pApp->GetProfileInt("ZpOpt", "fEncryption", FALSE);
	ZpOpt.fRecurse = pApp->GetProfileInt("ZpOpt", "fRecurse", 1);
	ZpOpt.fRepair = pApp->GetProfileInt("ZpOpt", "fRepair", 0);
	ZpOpt.fLevel = pApp->GetProfileInt("ZpOpt", "fLevel", 6) + 0x30;
}

void ChangeZipOptions()
{
	CConfigDlg dlg;
	
	dlg.m_stDate = ZpOpt.Date;
	//dlg.m_stRootDir = ZpOpt.szRootDir;
	dlg.m_stTempDir = ZpOpt.szTempDir;
	dlg.m_bTemp = ZpOpt.fTemp;
	dlg.m_bSuffix = ZpOpt.fSuffix;
	dlg.m_bEncrypt = ZpOpt.fEncrypt;
	dlg.m_bSystem = ZpOpt.fSystem;
	dlg.m_bVolume = ZpOpt.fVolume;
	dlg.m_bExtra = ZpOpt.fExtra;
	dlg.m_bNoDirEntries = ZpOpt.fNoDirEntries;
	dlg.m_bExcludeDate = ZpOpt.fExcludeDate;
	dlg.m_bIncludeDate = ZpOpt.fIncludeDate;
	dlg.m_bVerbose = ZpOpt.fVerbose;
	dlg.m_bQuiet = ZpOpt.fQuiet;
	dlg.m_bCRLF_LF = ZpOpt.fCRLF_LF;
	dlg.m_bLF_CRLF = ZpOpt.fLF_CRLF;
	dlg.m_bJunkDir = ZpOpt.fJunkDir;
	dlg.m_bGrow = ZpOpt.fGrow;
	dlg.m_bForce = ZpOpt.fForce;
	dlg.m_bMove = ZpOpt.fMove;
	dlg.m_bDeleteEntries = ZpOpt.fDeleteEntries;
	dlg.m_bUpdate = ZpOpt.fUpdate;
	dlg.m_bFreshen = ZpOpt.fFreshen;
	dlg.m_bJunkSFX = ZpOpt.fJunkSFX;
	dlg.m_bLatestTime = ZpOpt.fLatestTime;
	dlg.m_bComment = ZpOpt.fComment;
	dlg.m_bOffsets = ZpOpt.fOffsets;
	dlg.m_bPrivilege = ZpOpt.fPrivilege;
	dlg.m_bEncryption = ZpOpt.fEncryption;
	dlg.m_nRecurse = ZpOpt.fRecurse;
	dlg.m_nRepair = ZpOpt.fRepair;
	dlg.nLevel = ZpOpt.fLevel - 0x30;
	
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
	{
		lstrcpy(ZpOpt.Date, dlg.m_stDate);
		//lstrcpy(ZpOpt.szRootDir, dlg.m_stRootDir);
		lstrcpy(ZpOpt.szTempDir, dlg.m_stTempDir);
		ZpOpt.fTemp = dlg.m_bTemp;
		ZpOpt.fSuffix = dlg.m_bSuffix;
		ZpOpt.fEncrypt = dlg.m_bEncrypt;
		ZpOpt.fSystem = dlg.m_bSystem;
		ZpOpt.fVolume = dlg.m_bVolume;
		ZpOpt.fExtra = dlg.m_bExtra;
		ZpOpt.fNoDirEntries = dlg.m_bNoDirEntries;
		ZpOpt.fExcludeDate = dlg.m_bExcludeDate;
		ZpOpt.fIncludeDate = dlg.m_bIncludeDate;
		ZpOpt.fVerbose = dlg.m_bVerbose;
		ZpOpt.fQuiet = dlg.m_bQuiet;
		ZpOpt.fCRLF_LF = dlg.m_bCRLF_LF;
		ZpOpt.fLF_CRLF = dlg.m_bLF_CRLF;
		ZpOpt.fJunkDir = dlg.m_bJunkDir;
		ZpOpt.fGrow = dlg.m_bGrow;
		ZpOpt.fForce = dlg.m_bForce;
		ZpOpt.fMove = dlg.m_bMove;
		ZpOpt.fDeleteEntries = dlg.m_bDeleteEntries;
		ZpOpt.fUpdate = dlg.m_bUpdate;
		ZpOpt.fFreshen = dlg.m_bFreshen;
		ZpOpt.fJunkSFX = dlg.m_bJunkSFX;
		ZpOpt.fLatestTime = dlg.m_bLatestTime;
		ZpOpt.fComment = dlg.m_bComment;
		ZpOpt.fOffsets = dlg.m_bOffsets;
		ZpOpt.fPrivilege = dlg.m_bPrivilege;
		ZpOpt.fEncryption = dlg.m_bEncryption;
		ZpOpt.fRecurse = dlg.m_nRecurse;
		ZpOpt.fRepair = dlg.m_nRepair;
		ZpOpt.fLevel = dlg.nLevel+ 0x30;
	}
}

void SaveZipOptions()
{
	CWinApp* pApp = AfxGetApp();
	
	pApp->WriteProfileString("ZpOpt", "Date", ZpOpt.Date);
	//pApp->WriteProfileString("ZpOpt", "szRootDir", ZpOpt.szRootDir);
	pApp->WriteProfileString("ZpOpt", "szTempDir", ZpOpt.szTempDir);
	pApp->WriteProfileInt("ZpOpt", "fTemp", ZpOpt.fTemp);
	pApp->WriteProfileInt("ZpOpt", "fSuffix", ZpOpt.fSuffix);
	pApp->WriteProfileInt("ZpOpt", "fEncrypt", ZpOpt.fEncrypt);
	pApp->WriteProfileInt("ZpOpt", "fSystem", ZpOpt.fSystem);
	pApp->WriteProfileInt("ZpOpt", "fVolume", ZpOpt.fVolume);
	pApp->WriteProfileInt("ZpOpt", "fExtra", ZpOpt.fExtra);
	pApp->WriteProfileInt("ZpOpt", "fNoDirEntries", ZpOpt.fNoDirEntries);
	pApp->WriteProfileInt("ZpOpt", "fExcludeDate", ZpOpt.fExcludeDate);
	pApp->WriteProfileInt("ZpOpt", "fIncludeDate", ZpOpt.fIncludeDate);
	pApp->WriteProfileInt("ZpOpt", "fVerbose", ZpOpt.fVerbose);
	pApp->WriteProfileInt("ZpOpt", "fQuiet", ZpOpt.fQuiet);
	pApp->WriteProfileInt("ZpOpt", "fCRLF_LF", ZpOpt.fCRLF_LF);
	pApp->WriteProfileInt("ZpOpt", "fLF_CRLF", ZpOpt.fLF_CRLF);
	pApp->WriteProfileInt("ZpOpt", "fJunkDir", ZpOpt.fJunkDir);
	pApp->WriteProfileInt("ZpOpt", "fGrow", ZpOpt.fGrow);
	pApp->WriteProfileInt("ZpOpt", "fForce", ZpOpt.fForce);
	pApp->WriteProfileInt("ZpOpt", "fMove", ZpOpt.fMove);
	pApp->WriteProfileInt("ZpOpt", "fDeleteEntries", ZpOpt.fDeleteEntries);
	pApp->WriteProfileInt("ZpOpt", "fUpdate", ZpOpt.fUpdate);
	pApp->WriteProfileInt("ZpOpt", "fFreshen", ZpOpt.fFreshen);
	pApp->WriteProfileInt("ZpOpt", "fJunkSFX", ZpOpt.fJunkSFX);
	pApp->WriteProfileInt("ZpOpt", "fLatestTime", ZpOpt.fLatestTime);
	pApp->WriteProfileInt("ZpOpt", "fComment", ZpOpt.fComment);
	pApp->WriteProfileInt("ZpOpt", "fOffsets", ZpOpt.fOffsets);
	pApp->WriteProfileInt("ZpOpt", "fPrivilege", ZpOpt.fPrivilege);
	//pApp->WriteProfileInt("ZpOpt", "fEncryption", ZpOpt.fEncryption);
	pApp->WriteProfileInt("ZpOpt", "fRecurse", ZpOpt.fRecurse);
	pApp->WriteProfileInt("ZpOpt", "fRepair", ZpOpt.fRepair);
	pApp->WriteProfileInt("ZpOpt", "fLevel", ZpOpt.fLevel - 0x30);
}

void CreateStatusDlg()
{
	pDlg = new CStatusDlg();
	pDlg->Create();
	pDlg->m_ctrlOk.EnableWindow(FALSE);
	pDlg->m_ctrlStatus.EnableWindow(FALSE);
}

void DestroyStatusDlg()
{
	ASSERT(pDlg);
	
	pDlg->UpdateData();
	pDlg->m_ctrlStatus.EnableWindow();
	pDlg->m_ctrlOk.EnableWindow();
	if (pDlg->m_bClose)
	{
		pDlg->DestroyWindow();
		pDlg = NULL;
	}
}

int WINAPI UpdateStatusDlg(char far *buf, unsigned long size)
{
	ASSERT(pDlg);
	
	TRACE("%s", buf);
	
	pDlg->m_ctrlStatus.SetSel(-1,-1);
	pDlg->m_ctrlStatus.ReplaceSel(buf);
	
	MSG msg;
	while (PeekMessage(&msg, (HWND)NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	
	return (unsigned int) size;
}

int WINAPI PasswordPrompt(LPSTR p, int n, LPCSTR m, LPCSTR name)
{
	CPasswordDlg dlg;
	
	dlg.m_stPrompt = m;
	
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
		lstrcpy(p, dlg.m_stPassword);
	else if (nResponse == IDCANCEL)
		lstrcpy(p, "\0");
	
	return 0;
}

int WINAPI CommentPrompt(char far *szBuf)
{
	CCommentDlg dlg;
	
	dlg.m_stPrompt = "enter new zip file comment: ";
	dlg.m_stComment = szBuf;
	
	int nResponse = dlg.DoModal();
	
	if (nResponse == IDOK)
		lstrcpy(szBuf, dlg.m_stComment);
	else if (nResponse == IDCANCEL)
		lstrcpy(szBuf, "\0");
	
	return TRUE;
}

char *GetFileName(char *Path)
{
	char *Pos;
	
	if((Pos = strchr(Path, ':')) != NULL)
		Path = Pos + 1;
	
	if((Pos = strrchr(Path, '\\')) != NULL)
		Path = Pos + 1;
	
	if((Pos = strrchr(Path, '/')) != NULL)
		Path = Pos + 1;
	
	return(Path);
}

void RemoveFileName(char *Path, char *Buf)
{
	char *Pos;
	
	strcpy(Buf, Path);
	
	if((Pos = strrchr(Buf, '/')) != NULL)
		*Pos = '\0';
	else if((Pos = strrchr(Buf, '\\')) != NULL)
	{
		if((Pos == Buf) || 
			((Pos != Buf) && (*(Pos - 1) != ':')))
			*Pos = '\0';
	}
	return;
}

int DoArc(CString stFileName, HDROP hDropInfo)
{
	ASSERT(fpZpSetOptions);
	ASSERT(fpZpArchive);
	
	char szFileList[BUFFSIZE], szPath[_MAX_PATH], szRoot[_MAX_PATH];
	char **index, *sz;
	int retcode, i, cc;
	
	ZpZCL.argc = DragQueryFile(hDropInfo, -1, NULL, 0);
	ZpZCL.lpszZipFN = stFileName.GetBuffer(_MAX_PATH);
	
	index = (char **)szFileList;
	cc = (sizeof(char *) * ZpZCL.argc);
	sz = szFileList + cc;
	
	for (i = 0; i < ZpZCL.argc; i++)
	{
		char szFile[_MAX_PATH];
		
		DragQueryFile(hDropInfo, i, szPath, _MAX_PATH);
		
		lstrcpy(szFile, GetFileName(szPath));
		cc = lstrlen(szFile);
		lstrcpy(sz, szFile);
		index[i] = sz;
		sz += (cc + 1);
	}
	ZpZCL.FNV = (char **)szFileList;
	
	RemoveFileName(szPath, szRoot);
	ZpOpt.szRootDir = szRoot;
	
	if (!ZpOpt.fEncryption && ZpOpt.fEncrypt)
	{
		AfxMessageBox("ZIP32.DLL ɂ̈ÍT|[gĂ܂B");
		ZpOpt.fEncrypt = FALSE;
	}
	
	(*fpZpSetOptions)(&ZpOpt);
	retcode = (*fpZpArchive)(ZpZCL);
	TRACE("----------------\n");
	
	stFileName.ReleaseBuffer();
	return retcode;
}
